/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef abydos_h
#define abydos_h

#include <cairo.h>
#include <stddef.h>

#include "abydos-version.h"

/**
 * ABYDOS_FEATURE_SQUARE_PIXELS
 *
 * The image has perfectly square pixels. Meaning that it doesn't need to be
 * scaled in order to be displayed with correct aspect ratio.
 **/
#define ABYDOS_FEATURE_SQUARE_PIXELS (1<<0)
/**
 * ABYDOS_FEATURE_SCALABLE_SIZE
 *
 * The image (or some aspect of it) can be scaled up. This is typical for
 * vector images.
 **/
#define ABYDOS_FEATURE_SCALABLE_SIZE (1<<1)
/**
 * ABYDOS_FEATURE_SCALABLE_TIME
 *
 * The image format has the ability to (at least potentially) create images
 * between the defined frames. It is also set if the resulting image may
 * differ depending on other variables (such as the current clock time).
 */
#define ABYDOS_FEATURE_SCALABLE_TIME (1<<2)

/**
 * ABYDOS_PREFER_SHARE
 *
 * Prefer to get a shared object reference. This is always fastest but
 * alterations to the object may or may not also affect the next reference
 * you request. If you intend to alter the object and want to be sure you
 * can request more references to (unaltered) objects later, use
 * ABYDOS_PREFER_COPY or ABYDOS_PREFER_TRANSFER instad.
 **/
#define ABYDOS_PREFER_SHARE    (1<<0)
/**
 * ABYDOS_PREFER_COPY
 *
 * Prefer to get a copy of the object. This is always safe then the object
 * needs to be altered. But it may also be slowest if a copy doesn't
 * actually need to be made. Consider using ABYDOS_PREFER_TRANSFER instead
 * if you consider it unlikely you will need more references to the
 * (unaltered) object later.
 **/
#define ABYDOS_PREFER_COPY     (2<<0)
/**
 * ABYDOS_PREFER_TRANSFER
 *
 * Prefer to take over the ownership of the object. This is always safe then
 * the object needs to be alteter. But it may also be slowest if you need
 * another copy later and it has to be recreated. Consider using
 * ABYDOS_PREFER_COPY instead if you consider it likely you will need more
 * references to the (unaltered) object.
 **/
#define ABYDOS_PREFER_TRANSFER (3<<0)
#define ABYDOS_PREFER_MASK     (3<<0)
/**
 * ABYDOS_IGNORE_ASPECT_RATIO
 *
 * Ignore the aspect ratio and get a pixel by pixel copy of the image.
 */
#define ABYDOS_IGNORE_ASPECT_RATIO    (1<<2)
/**
 * ABYDOS_SHRINK_TO_ASPECT_RATIO
 *
 * Shrink the image to correct aspect ratio.
 */
#define ABYDOS_SHRINK_TO_ASPECT_RATIO (2<<2)
/**
 * ABYDOS_GROW_TO_ASPECT_RATIO
 *
 * Grow the image to correct aspect ratio.
 **/
#define ABYDOS_GROW_TO_ASPECT_RATIO   (3<<2)
#define ABYDOS_ASPECT_RATIO_MASK      (3<<2)

/**
 * abydos_t
 *
 * An abydos_t is an opaque type representing an image. It is both used for
 * loading the image then for rendering it. It has also an internal state
 * and can be used as an iterator in several ways. If you need several
 * independant iterators you can use abydos_clone(), which is cheap and
 * effectively only makes a copy of the state.
 **/
typedef struct _abydos_t abydos_t;

/**
 * abydos_info_t:
 * @version: Version of this structure supported by the runtime version of
 *           abydos. You can use the macro ABYDOS_INFO_HAS() to check if a
 *           member of #abydos_info_t is available at runtime.
 * @width: Width of the image.
 * @height: Height of the image.
 * @features: A set of features supported by the image. They can be:
 * - %ABYDOS_FEATURE_SQUARE_PIXELS
 * - %ABYDOS_FEATURE_SCALABLE_SIZE
 * - %ABYDOS_FEATURE_SCALABLE_TIME
 * @pixel_ratio: The aspect ratio of the pixels. The value is 1 is the
 *               pixels are perfectly square. If @pixel_ratio if greater
 *               than one the pixels are wider than they are high.
 *
 * Information about an image.
 **/
typedef struct _abydos_info_t {
    int version;
    int width;
    int height;
    int features;
    double pixel_ratio;
} abydos_info_t;
#define ABYDOS_INFO_VERSION sizeof(abydos_info_t)
#define ABYDOS_INFO_HAS(p,f) (offsetof(abydos_info_t,f)<(p).version)

typedef void(*abydos_info_callback_t)(void *userdata,abydos_t *ar);
typedef void(*abydos_update_callback_t)(void *userdata,abydos_t *ar,cairo_rectangle_int_t *);

abydos_t *abydos_create(const char *mime_type);
abydos_t *abydos_clone(abydos_t *src);
abydos_t *abydos_reference(abydos_t *ar);
void abydos_destroy(abydos_t *ar);

int abydos_from_data(abydos_t *ar, const char *data, size_t len);
int abydos_from_file(abydos_t *ar, const char *filename);

void abydos_load_begin(abydos_t *ar,abydos_info_callback_t,abydos_update_callback_t,void *userdata);
int abydos_load_feed(abydos_t *ar, const char* data, size_t len);
int abydos_load_end(abydos_t *ar);

void abydos_get_info_V(abydos_t *ar,abydos_info_t *info,int);
#define abydos_get_info(a,b) abydos_get_info_V(a,b,ABYDOS_INFO_VERSION)
void abydos_get_size(abydos_t *ar,int *width,int *height,int flags);
const char *abydos_error(abydos_t *ar);

void abydos_render(abydos_t *ar,cairo_t *cr,int flags);
cairo_surface_t *abydos_get_image_surface(abydos_t *ar,int flags);
cairo_surface_t *abydos_get_image_surface_at_size_min(abydos_t *ar,int width,int height);
cairo_surface_t *abydos_get_image_surface_at_size_max(abydos_t *ar,int width,int height);

int abydos_get_page_count(abydos_t *ar);
int abydos_get_page(abydos_t *ar);
void abydos_set_page(abydos_t *ar,int page);

int abydos_get_layer_count(abydos_t *ar);
int abydos_get_layer_visibility(abydos_t *ar, int layer);
void abydos_set_layer_visibility(abydos_t *ar, int layer, int visibility);

int abydos_get_variant_count(abydos_t *ar);
void abydos_get_variant_size(abydos_t *ar,int variant,int *width, int *height);
int abydos_get_variant_at_size_min(abydos_t *ar,int width, int height);
int abydos_get_variant_at_size_max(abydos_t *ar,int width, int height);
void abydos_set_variant(abydos_t *ar,int variant);
void abydos_prefer_variant(abydos_t *,int width,int height);

int abydos_get_frame_count(abydos_t *ar);
double abydos_get_total_duration(abydos_t *ar);
int abydos_get_frame_at_time(abydos_t *ar,double sec);
void abydos_set_frame_at_time(abydos_t *ar,double sec);
double abydos_get_frame_duration(abydos_t *ar,int frame);
int abydos_get_frame(abydos_t *);
double abydos_get_time(abydos_t *);
void abydos_set_frame(abydos_t *,int frame);
cairo_bool_t abydos_set_time(abydos_t *,double sec);
void abydos_next_frame(abydos_t *);
cairo_bool_t abydos_add_time(abydos_t *ar,double sec);
double abydos_get_delay(abydos_t *);


abydos_t *abydos_create_from_file(const char *mime_type, const char *filename);
cairo_surface_t *abydos_load(const char *mime_type, const char *filename);
cairo_status_t abydos_write_to_png(abydos_t *, const char *filename);
cairo_status_t abydos_write_to_svg(abydos_t *, const char *filename);
cairo_status_t abydos_write_to_webp(abydos_t *,const char *filename);
cairo_bool_t abydos_can_animate(abydos_t *);

cairo_status_t abydos_write_to_apng(abydos_t *ar, const char *filename);
cairo_status_t abydos_write_to_mng(abydos_t *ar, const char *filename);

#endif /* abydos_h */
